import styled from "styled-components";
import { InputHTMLAttributes, useMemo, useState } from "react";
import { XIcon } from "@assets/Icon";

export type InputStatus = "DEFAULT" | "ERROR" | "FOCUS";

export type InputProps = {
  type?: "date" | "number" | "text" | "password";
  name?: string;
  value: string;
  onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
  onError?: () => boolean;
  onReset?: () => void;
  disable?: boolean;
  enableButton?: boolean;
  attributes?: InputHTMLAttributes<HTMLInputElement>;
  placeholder?: string;
};

export type InputProps2 = InputHTMLAttributes<HTMLInputElement> & {
  onClear?: () => void;
  onError?: () => boolean;
};

export const Input = ({
  onClear,
  onError = () => false,
  ...attributes
}: InputProps2) => {
  const [isFocus, setIsFocus] = useState(false);
  const status: InputStatus = useMemo(() => {
    if (onError()) return "ERROR";
    if (isFocus) return "FOCUS";
    return "DEFAULT";
  }, [isFocus, onError]);

  return (
    <InputContainer id="common-input" data-active={status}>
      <input
        {...attributes}
        onFocus={(e) => {
          setIsFocus(true);
          attributes.onFocus && attributes.onFocus(e);
        }}
        onBlur={(e) => {
          setIsFocus(false);
          attributes.onBlur && attributes.onBlur(e);
        }}
      />

      {onClear && attributes.value && !attributes.disabled && (
        <ClearButton
          onClick={() => {
            setIsFocus(false);
            onClear();
          }}
          className="common-input__clear-button"
        >
          <XIcon />
        </ClearButton>
      )}
    </InputContainer>
  );
};

const InputContainer = styled.div`
  width: 100%;
  height: 100%;
    max-height:40px;
  position: relative;
  background-color: #fff;
  border: 1px solid lightgray;

  input {
    width: 100%;
    height: 100%;
    padding: 0 16px;
    background: none;
    border: none;
    outline: none;

    &:disabled {
      background-color: lightgray;
      color: gray;
    }

    &::placeholder {
      color: lightgray;
    }
  }

  &[data-active="DEFAULT"] {
    border-color: lightgray;
  }

  &[data-active="ERROR"] {
    border-color: red;
  }

  &[data-active="FOCUS"] {
    border-color: blue;
  }
`;

export const ClearButton = styled.button`
  width: 16px;
  height: 16px;
  display: flex;
  align-items: center;
  justify-content: center;
  position: absolute;
  top: 50%;
  right: 16px;
  transform: translate(0, -50%);
  background-color: lightgray;
  border: 1px solid lightgray;
  border-radius: 50%;
  cursor: pointer;

  svg {
    width: 12px;
    height: 12px;
  }
`;
