import React, { useEffect, useMemo, useRef, useState } from "react";
import { InputStatus } from "../Input";
import { InputStyle } from "../Input/Input.styled";
import { blue, gray, red } from "../../../core/colors/colors";
import { CommonDropdownTypes } from "@src/components/Dropdown/DropdownTypes";
import { SearchDropdownInputContainer, SearchInputFilterValueContainer } from "./SearchDropdownInput.styled";

export type SearchDropdownInputProps<T> = CommonDropdownTypes<T> & {
  type?: "date" | "number" | "text" | "password";
  value: string;
  onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
  onBlur?: (e: React.FocusEvent<HTMLInputElement, Element>) => void;
  onError?: () => boolean;
};

const SearchDropdownInput = <T extends {}>({
  type = "text",
  value,
  options,
  optionKey,
  optionLabel,
  onChange, // input
  onBlur, // input
  onClick, // dropdown
  onError = () => false,
  disable = false,
  placeholder,
}: SearchDropdownInputProps<T>) => {
  const [isFocus, setIsFocus] = useState<boolean>(false);
  const [filteredOptions, setFilteredOptions] = useState<T[]>([]);
  const status: InputStatus = useMemo(() => {
    if (onError()) return "ERROR";
    if (isFocus) return "FOCUS";
    return "DEFAULT";
  }, [isFocus, onError]);
  const searchInputRef = useRef<HTMLDivElement>(null);

  const valueToShow = filteredOptions?.length !== 0 ? filteredOptions : options;

  const style: { [key in InputStatus]: { [style in "borderColor"]: string } } =
    {
      DEFAULT: {
        borderColor: gray[300],
      },
      ERROR: {
        borderColor: red[500],
      },
      FOCUS: {
        borderColor: blue[500],
      },
    };

  const onChangeSearch = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFilteredOptions(
      options.filter((v) =>
        optionLabel(v)
          .replace(/\s| /gi, "")
          .toLowerCase()
          .includes(e.currentTarget.value.replace(/\s| /gi, "").toLowerCase())
      )
    );
  };

  useEffect(() => {
    const outsideClick = (e: MouseEvent) => {
      if (
        searchInputRef.current &&
        !searchInputRef.current.contains(e.target as Node)
      ) {
        setIsFocus(false);
      }
    };
    window.addEventListener("mousedown", outsideClick);
    return () => window.removeEventListener("mousedown", outsideClick);
  }, [searchInputRef]);

  return (
    <SearchDropdownInputContainer ref={searchInputRef}>
      <InputStyle
        type={type}
        value={value}
        onChange={(e) => {
          onChange(e);
          onChangeSearch(e);
        }}
        onBlur={(e: React.FocusEvent<HTMLInputElement, Element>) => {
          onBlur && onBlur(e);
          setIsFocus(false);
        }}
        onFocus={() => setIsFocus(true)}
        onError={onError}
        disabled={disable}
        placeholder={placeholder}
        style={{ ...style[status] }}
      />
      {isFocus && !disable && (
        <SearchInputFilterValueContainer className="dropdown_options">
          {valueToShow.map((option) => (
            <button
              key={`search_input_filter_value_${optionKey(option)}`}
              className={`search_input_filter_value_${optionKey(option)}`}
              onClick={() => {
                onClick(option);
                setIsFocus(false);
              }}
            >
              <p>{optionLabel(option)}</p>
            </button>
          ))}
        </SearchInputFilterValueContainer>
      )}
    </SearchDropdownInputContainer>
  );
};

export default SearchDropdownInput;
