import styled from "styled-components";
import { ToastItem, ToastType } from "../utils/toaster";
import { ToastPosition } from "../index";
import { useEffect, useMemo, useRef } from "react";
import {
  CheckIcon,
  ExclamationMarkIcon,
  InfoCircleIcon,
  XIcon,
} from "@src/assets/Icon";

export type ToastProps = {
  toast: ToastItem;
  duration: number;
  position: ToastPosition;
  offsetY: number;
  addHeightOfToast: (toastId: ToastItem["id"], height: number) => void;
  removeToast: (toastId: ToastItem["id"]) => void;
};

const TRANSITION_DURATION = 240;

const Toast = ({
  toast,
  duration,
  position,
  offsetY,
  addHeightOfToast,
  removeToast,
}: ToastProps) => {
  const toastRef = useRef<HTMLDivElement>(null);

  const toastDurationWithTransition = useMemo(
    () => duration + TRANSITION_DURATION,
    [duration]
  );

  const icons: { [key in ToastType]: JSX.Element } = {
    ERROR: <XIcon />,
    INFOMATION: <InfoCircleIcon />,
    SUCCESS: <CheckIcon />,
    WARNING: <ExclamationMarkIcon />,
  };

  useEffect(() => {
    const timeoutForRemove = setTimeout(() => {
      removeToast(toast.id);
    }, toastDurationWithTransition);
    const opacityTransitionTimeout = setTimeout(() => {
      if (toastRef.current) {
        toastRef.current.style.opacity = "0";
      }
    }, duration); // transition 전까지의 duration만큼만 대기
    return () => {
      clearTimeout(timeoutForRemove);
      clearTimeout(opacityTransitionTimeout);
    };
  }, [removeToast, toast.id, toastDurationWithTransition, duration]);

  useEffect(() => {
    if (toastRef.current) {
      addHeightOfToast(toast.id, toastRef.current.clientHeight);
    }
  }, [toast, addHeightOfToast]);

  return (
    <Container
      ref={toastRef}
      className={`common-toast ${toast.type.toLowerCase()}`}
      style={{
        top: `${position === "top" && 0}`,
        bottom: `${position === "bottom" && 0}`,
        transform: `translate(-50%, ${
          position === "top" ? offsetY : -offsetY
        }px)`,
      }}
    >
      <IconContainer className="common-toast__icon-container">
        {icons[toast.type]}
      </IconContainer>
      <TextContainer className="common-toast__text-container">
        <p className="title">{toast.title}</p>
        {toast.description && (
          <p className="description">{toast.description}</p>
        )}
      </TextContainer>
      <RemoveButton
        onClick={() => removeToast(toast.id)}
        className="common-toast__remove-button"
      >
        <XIcon />
      </RemoveButton>
    </Container>
  );
};

const Container = styled.div`
  width: 40vw;
  max-width: 100%;

  display: flex;
  align-items: center;
  justify-content: space-between;
  gap: 16px;

  position: absolute;
  left: 50%;

  padding: 12px 16px;

  background-color: #fff;
  border: 1px solid lightgray;
  border-radius: 8px;
  box-shadow: rgba(149, 157, 165, 0.2) 0px 8px 24px;

  opacity: 1;
  transition: transform ${TRANSITION_DURATION}ms,
    opacity ${TRANSITION_DURATION}ms;

  pointer-events: auto;

  &.error {
    background-color: #ffe6d6;
    border-color: #ff4332;

    .common-toast__icon-container svg {
      stroke: #ff4332;
    }

    .title {
      color: #ff4332;
    }
  }

  &.infomation {
    background-color: #d6f6fe;
    border-color: #36a5f9;

    .common-toast__icon-container svg {
      stroke: #36a5f9;
    }

    .title {
      color: #36a5f9;
    }
  }

  &.success {
    background-color: #eefcd2;
    border-color: #6fcc24;

    .common-toast__icon-container svg {
      stroke: #6fcc24;
    }

    .title {
      color: #6fcc24;
    }
  }

  &.warning {
    background-color: #fff6d4;
    border-color: #ffb42b;

    .common-toast__icon-container svg {
      stroke: #ffb42b;
    }

    .title {
      color: #ffb42b;
    }
  }
`;

const IconContainer = styled.div`
  width: 24px;
  height: 24px;

  svg {
    width: 100%;
    height: 100%;
    stroke-width: 2px;
  }
`;

const TextContainer = styled.div`
  display: flex;
  flex-direction: column;
  gap: 8px;
  margin-right: auto;

  .title {
    font-weight: 600;
  }

  .description {
    font-size: 14px;
    color: gray;
  }
`;

const RemoveButton = styled.button`
  width: 16px;
  height: 16px;
  background: none;
  border: 0;

  svg {
    width: 100%;
    height: 100%;
    stroke-width: 2px;
    stroke: gray;
  }
`;

export default Toast;
